@@ -20,6 +20,7 @@
 #include <string.h>
 
 #include <thunar-vfs/thunar-vfs.h>
+#include <libexif/exif-data.h>
 
 #include "navigator.h"
 
@@ -57,6 +58,7 @@ struct _RsttoNavigatorEntry
     GdkPixbuf           *pixbuf;
     gdouble              scale;
     gboolean             fit_to_screen;
+    ExifData            *exif_data;
     GdkPixbufRotation    rotation;
     gboolean             h_flipped;
     gboolean             v_flipped;
@@ -295,6 +297,12 @@ rstto_navigator_set_running (RsttoNavigator *navigator, gboolean running)
     }
 }
 
+gboolean
+rstto_navigator_is_running (RsttoNavigator *navigator)
+{
+    return navigator->running;
+}
+
 RsttoNavigatorEntry *
 rstto_navigator_get_file (RsttoNavigator *navigator)
 {
@@ -326,7 +334,7 @@ rstto_navigator_get_nth_file (RsttoNavigator *navigator, gint n)
     return g_list_nth_data(navigator->file_list, n);
 }
 
-void
+gint
 rstto_navigator_add (RsttoNavigator *navigator, RsttoNavigatorEntry *entry)
 {
     if(navigator->file_iter)
@@ -346,6 +354,7 @@ rstto_navigator_add (RsttoNavigator *navigator, RsttoNavigatorEntry *entry)
                       NULL);
     }
     g_signal_emit(G_OBJECT(navigator), rstto_navigator_signals[RSTTO_NAVIGATOR_SIGNAL_NEW_ENTRY], 0, g_list_index(navigator->file_list, entry), entry, NULL);
+    return g_list_index(navigator->file_list, entry);
 }
 
 void
@@ -433,6 +442,20 @@ rstto_navigator_set_file (RsttoNavigator *navigator, gint n)
     }
 }
 
+void
+rstto_navigator_set_is_album (RsttoNavigator *navigator, gboolean album)
+{
+    navigator->album = album;
+}
+
+gboolean
+rstto_navigator_get_is_album (RsttoNavigator *navigator)
+{
+    return navigator->album;
+}
+
+/* Callbacks */
+
 static gboolean
 cb_rstto_navigator_running(RsttoNavigator *navigator)
 {
@@ -475,6 +498,63 @@ rstto_navigator_entry_new (ThunarVfsInfo *info)
         entry = g_new0(RsttoNavigatorEntry, 1);
 
         entry->info = info;
+        entry->exif_data = exif_data_new_from_file(filename);
+        
+        ExifEntry *exifentry = exif_data_get_entry(entry->exif_data, EXIF_TAG_ORIENTATION);
+        if (exifentry)
+        {
+            gchar *val = g_new0(gchar, 20);
+            exif_entry_get_value(exifentry, val, 20);
+            if (!strcmp(val, "top - left"))
+            {
+                entry->v_flipped = FALSE;
+                entry->h_flipped = FALSE;
+                entry->rotation = GDK_PIXBUF_ROTATE_NONE;
+            }
+            if (!strcmp(val, "top - right"))
+            {
+                entry->v_flipped = FALSE;
+                entry->h_flipped = TRUE;
+                entry->rotation = GDK_PIXBUF_ROTATE_NONE;
+            }
+            if (!strcmp(val, "bottom - left"))
+            {
+                entry->v_flipped = TRUE;
+                entry->h_flipped = FALSE;
+                entry->rotation = GDK_PIXBUF_ROTATE_NONE;
+            }
+            if (!strcmp(val, "bottom - right"))
+            {
+                entry->v_flipped = FALSE;
+                entry->h_flipped = FALSE;
+                entry->rotation = GDK_PIXBUF_ROTATE_UPSIDEDOWN;
+            }
+            if (!strcmp(val, "right - top"))
+            {
+                entry->v_flipped = FALSE;
+                entry->h_flipped = FALSE;
+                entry->rotation = GDK_PIXBUF_ROTATE_CLOCKWISE;
+            }
+            if (!strcmp(val, "right - bottom"))
+            {
+                entry->v_flipped = FALSE;
+                entry->h_flipped = TRUE;
+                entry->rotation = GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE;
+            }
+            if (!strcmp(val, "left - top"))
+            {
+                entry->v_flipped = FALSE;
+                entry->h_flipped = TRUE;
+                entry->rotation = GDK_PIXBUF_ROTATE_CLOCKWISE;
+            }
+            if (!strcmp(val, "left - bottom"))
+            {
+                entry->v_flipped = FALSE;
+                entry->h_flipped = FALSE;
+                entry->rotation = GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE;
+            }
+            g_free(val);
+        }
 
         g_free(filename);
     }
@@ -513,6 +593,10 @@ rstto_navigator_entry_free(RsttoNavigatorEntry *nav_entry)
     {
         g_object_unref(nav_entry->thumb);
     }
+    if(nav_entry->exif_data)
+    {   
+        exif_data_free(nav_entry->exif_data);
+    }
     thunar_vfs_info_unref(nav_entry->info);
     g_free(nav_entry);
 }
@@ -547,7 +631,21 @@ rstto_navigator_entry_get_pixbuf(RsttoNavigatorEntry *entry)
     if(!entry->pixbuf)
     {
         gchar *filename = thunar_vfs_path_dup_string(entry->info->path);
-        entry->pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
+        GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
+        entry->pixbuf = gdk_pixbuf_rotate_simple(pixbuf, entry->rotation);
+        gdk_pixbuf_unref(pixbuf);
+        if (entry->v_flipped)
+        {
+            pixbuf = entry->pixbuf;
+            entry->pixbuf = gdk_pixbuf_flip(pixbuf, FALSE);
+            gdk_pixbuf_unref(pixbuf);
+        }
+        if (entry->v_flipped)
+        {
+            pixbuf = entry->pixbuf;
+            entry->pixbuf = gdk_pixbuf_flip(pixbuf, TRUE);
+            gdk_pixbuf_unref(pixbuf);
+        }
         g_free(filename);
     }
     return entry->pixbuf;
@@ -599,3 +697,10 @@ rstto_navigator_entry_set_fit_to_screen (RsttoNavigatorEntry *entry, gboolean ft
     entry->fit_to_screen = fts;
 }
 
+
+ExifData *
+rstto_navigator_entry_get_exif_data (RsttoNavigatorEntry *entry)
+{
+    return entry->exif_data;
+}
+
