@@ -50,6 +50,7 @@ struct _RsttoNavigator
     ThunarVfsPath      *path;
     GList              *file_list;
     GList              *file_iter;
+    gboolean            album;
     gint                old_position;
     gboolean            running;
     gint                timeout;
@@ -75,6 +76,7 @@ void       rstto_navigator_jump_last (RsttoNavigator *navigator);
 
 void       rstto_navigator_set_running (RsttoNavigator *navigator,
                                         gboolean running);
+gboolean   rstto_navigator_is_running (RsttoNavigator *navigator);
 void
 rstto_navigator_set_timeout (RsttoNavigator *nav, gint);
 
@@ -86,7 +88,7 @@ rstto_navigator_get_position(RsttoNavigator *navigator);
 gint
 rstto_navigator_get_old_position(RsttoNavigator *navigator);
 
-void
+gint
 rstto_navigator_add (RsttoNavigator *navigator, RsttoNavigatorEntry *entry);
 void
 rstto_navigator_remove (RsttoNavigator *navigator, RsttoNavigatorEntry *entry);
@@ -99,6 +101,10 @@ rstto_navigator_set_file (RsttoNavigator *navigator, gint n);
 void
 rstto_navigator_clear (RsttoNavigator *navigator);
 void
+rstto_navigator_set_is_album (RsttoNavigator *navigator, gboolean album);
+gboolean
+rstto_navigator_get_is_album (RsttoNavigator *navigator);
+void
 rstto_navigator_set_entry_rotation (RsttoNavigator *navigator, RsttoNavigatorEntry *entry, GdkPixbufRotation rotation);
 
 RsttoNavigatorEntry *
@@ -126,6 +132,8 @@ void
 rstto_navigator_flip_entry(RsttoNavigator *navigator, RsttoNavigatorEntry *entry, gboolean horizontal);
 GdkPixbuf *
 rstto_navigator_entry_get_thumb(RsttoNavigatorEntry *entry, gint max_size);
+ExifData *
+rstto_navigator_entry_get_exif_data (RsttoNavigatorEntry *entry);
 
 G_END_DECLS
 
